require 'forwardable'

require 'rubygems'
require 'json'
require 'time'
require 'net/http'
require 'digest/md5'
require 'pathname'

module Bazil
  class REST
    extend Forwardable

    def initialize(http, options = {})
      @http = http
      @api_root = options[:api_root] || '/'
      @api_root += '/' unless @api_root.end_with? '/'
    end

    def_delegators :@http, :read_timeout, :read_timeout=

    def set_api_keys(key, secret)
      @api_key = key
      @secret_key = secret
      true
    end

    def get(uri)
      uri = gen_path(uri)
      uri, header = add_api_signature(uri, nil)
      @http.get(uri, header)
    rescue Errno::ECONNREFUSED => e
      raise_error('GET')
    end

    def put(uri, data, header = {})
      uri = gen_path(uri)
      uri, header = add_api_signature(uri, data, header)
      @http.put(uri, data, header)
    rescue Errno::ECONNREFUSED => e
      raise_error('PUT')
    end

    def post(uri, data, header = {})
      uri = gen_path(uri)
      uri, header = add_api_signature(uri, data, header)
      @http.post(uri, data, header)
    rescue Errno::ECONNREFUSED => e
      raise_error('POST')
    end

    def delete(uri)
      uri = gen_path(uri)
      uri, header = add_api_signature(uri, nil)
      @http.delete(uri, header)
    rescue Errno::ECONNREFUSED => e
      raise_error('DELETE')
    end

    private
    def gen_path(path)
      # Pathname#+ cannot append absolute path, so use String#+ then canonicalize it
      Pathname.new(@api_root + path).cleanpath.to_s
    end

    def add_api_signature(uri, data, header = {})
      return uri, header unless @api_key and @secret_key

      base,param = uri.split('?',2)
      current_time = Time.now.httpdate

      signature = ''
      signature = data.gsub(/\s/, '') if data
      parameters = (param || "").split('&')
      parameters << "api_key=#{@api_key}"
      signature << parameters.sort.join()
      signature << current_time
      signature << @secret_key
      parameters << "api_signature=#{Digest::MD5.hexdigest(signature)}"
      base << '?'
      base << parameters.join('&')

      return base, header.merge({'Date' => current_time})
    end

    def raise_error(method)
      raise ConnectionError.new(method, @http.address, @http.port)
    end
  end # class Rest
end # module Bazil
